<?php

namespace Drupal\views_extra_exposed_form_blocks\Plugin\views\display_extender;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\views\Attribute\ViewsDisplayExtender;
use Drupal\views\Plugin\views\display_extender\DisplayExtenderPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Display extender for support for extra exposed blocks.
 *
 * This alters the request query so that exposed filter values from an extra
 * exposed block work with the exposed filters on the current display.
 *
 * In particular:
 * - If the extra block has exposed filters which have been overridden to be
 *   single-valued where the linked display's equivalent is multi-valued, then:
 *   - An 'All' query value must be removed.
 *   - A single scalar query value must be converted to a single value in an
 *     array value.
 */
#[ViewsDisplayExtender(
  id: 'views_extra_exposed_form_block',
  title: new TranslatableMarkup('Views Extra Exposed Form Block'),
  short_title: new TranslatableMarkup('Views Extra Exposed Form Block'),
  help: new TranslatableMarkup('Provides support for views that use the Extra Exposed Form Block display type.'),
)]
class ViewsExtraExposedFormBlock extends DisplayExtenderPluginBase {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('request_stack'),
    );
  }

  /**
   * Creates a ViewsExtraExposedFormBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    RequestStack $request_stack,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->requestStack = $request_stack;
  }

   /**
    * {@inheritdoc}
    */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['filter_query_multiple'] = ['default' => FALSE];

    return $options;
  }


  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $section = $form_state->get('section');

    switch ($section) {
      case 'exposed_form':
        $form['filter_query_multiple'] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Process query string parameters for Extra Exposed Form Block exposed filters'),
          '#description' => $this->t('Enable this on a display that is linked to an Extra Exposed Form Block display. This alters query string parameters for exposed filters which are single-valued in the extra block but multi-valued on the linked display.'),
        ];

        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state);

    if ($form_state->get('section') == 'exposed_form') {
      // Save options for this plugin.
      foreach ($this->defineOptions() as $option_name => $definition) {
        $this->options[$option_name] = $form_state->getValue($option_name);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preExecute() {
    $filters = $this->view->getHandlers('filter');

    // Re-key the exposed filters by their query identifier. These are enforced
    // to be unique by Views.
    $filters_by_query_key = [];
    foreach ($filters as $filter) {
      // Skip a filter that is not exposed.
      if (empty($filter['exposed'])) {
        continue;
      }

      $filters_by_query_key[$filter['expose']['identifier']] = $filter;
    }

    // Get the current request so we can alter the query parameters.
    $request = $this->requestStack->getCurrentRequest();

    foreach ($request->query as $query_key => $query_value) {
      // Skip a query parameter that is not for an exposed filter.
      if (!isset($filters_by_query_key[$query_key])) {
        continue;
      }

      // Skip an exposed filter that is not multiple-value.
      if (empty($filters_by_query_key[$query_key]['expose']['multiple'])) {
        continue;
      }

      // Remove an 'All' value coming from a single-valued exposed filter.
      // @todo Get the value of 'All' from Views config.
      if ($query_value == 'All') {
        $request->query->remove($query_key);
        continue;
      }

      // Query parameter value is already an array: nothing to do.
      if (is_array($query_value)) {
        continue;
      }

      // Alter the query parameter value so that an incoming value that is
      // from a single-valued exposed filter works with a multi-valued exposed
      // filter.
      $request->query->set($query_key, [$query_value => $query_value]);
    }
  }

}
